import { Observable } from './vendor/rxjs';
import { AnyMessage, INotificationObject, IRequestObject, IResponseObject, MappedMessage, MessageType } from './interface/message-objects';
import PublicInterface from '../rx-postmessenger';
import { IMessageFactory } from './interface/message-factory';
import IEventMap = PublicInterface.EventMap;
import IMessenger = PublicInterface.Messenger;
import IRequest = PublicInterface.Request;
import TypeLens = PublicInterface.TypeLens;
import { IMessageValidator } from './interface/message-validator';
import { IPostmessageAdapter } from './interface/postmessage-adapter';
/**
 * @class RxPostmessenger
 */
export declare class Messenger<MAP extends IEventMap = any> implements IMessenger {
    protected readonly messageFactory: IMessageFactory;
    protected readonly messageValidator: IMessageValidator;
    protected readonly adapter: IPostmessageAdapter;
    /**
     * Observable stream of all incoming messages that originate from remoteWindow with a
     * remoteOrigin url matching this.remoteOrigin.
     */
    readonly inboundMessages$: Observable<AnyMessage>;
    /** Filtered subset of inboundMessages$, emitting all messages of type 'notification' */
    readonly notifications$: Observable<INotificationObject>;
    /** Filtered subset of inboundMessages$, emitting all messages of type 'request' */
    readonly requests$: Observable<IRequestObject>;
    /** Filtered subset of inboundMessages$, emitting all messages of type 'response' */
    readonly responses$: Observable<IResponseObject>;
    constructor(messageFactory: IMessageFactory, messageValidator: IMessageValidator, adapter: IPostmessageAdapter);
    /**
     * @param {string} channel
     * @param {*} payload
     *
     * @return {Observable<object>}
     * @public
     */
    request<CH extends TypeLens.Out.Request.Channel<MAP>, REQ_PL extends TypeLens.Out.Request.RequestPayload<MAP, CH>, RES_PL extends TypeLens.Out.Request.ResponsePayload<MAP, CH>>(channel: CH, payload: REQ_PL): Observable<RES_PL>;
    /**
     * Sends a notification through given channel to the remote window, carrying given payload.
     *
     * @param {string} channel
     * @param {*} payload
     * @return {Messenger}
     * @public
     */
    notify<CH extends TypeLens.Out.Notification.Channel<MAP>, PL extends TypeLens.Out.Notification.Payload<MAP, CH>>(channel: CH, payload: PL): this;
    /**
     * Returns an Observable that emits the subset of inbound notification messages where their channel equals given
     * channel name.
     *
     * @param {string} channel
     * @return {Observable<object>}
     * @public
     */
    requests<CH extends TypeLens.In.Request.Channel<MAP>>(channel: CH): Observable<IRequest<MAP, CH>>;
    /**
     * Returns an Observable that emits the subset of inbound notification messages where their channel equals given
     * channel name.
     *
     * @param {string} channel
     * @return {Observable<*>}
     * @public
     */
    notifications<CH extends TypeLens.In.Notification.Channel<MAP>>(channel: CH): Observable<TypeLens.In.Notification.Payload<MAP, CH>>;
    /**
     * Sends a response through given channel to the remote window, carrying given payload.
     *
     * @param {string} requestId
     * @param channel
     * @param {*} payload
     * @return {Messenger}
     * @private
     */
    protected respond<T>(requestId: string, channel: string, payload: T): this;
    /**
     * Creates an Observable that completes after a single emission of the MessageEvent response for request of given
     * requestId. If no matching response is ever received, the Observable never completes.
     *
     * @param {string} requestId
     * @return {Observable<object>}
     * @private
     */
    protected createResponseObservable<T>(requestId: string): Observable<T>;
    /**
     * Returns an Observable emitting all inbound messages` of given type.
     *
     * Returns an Observable emitting a subset of MessageEvent objects emitted
     * by this.inboundMessages$, passing through all MessageEvent objects whose
     * data.type property matches given type.
     *
     * @param {string} type
     * @return {Observable<object>}
     * @private
     */
    protected messagesOfType<T extends MessageType>(type: T): Observable<MappedMessage<T>>;
}
