// -----------------------------------------------------------------------------
// Concrete imports
// -----------------------------------------------------------------------------
import { getObservable } from './index';
import { RxPostmessengerRequest } from './RxPostmessengerRequest';
/**
 * @class RxPostmessenger
 */
var Messenger = /** @class */ (function () {
    function Messenger(messageFactory, messageValidator, adapter) {
        var _this = this;
        this.messageFactory = messageFactory;
        this.messageValidator = messageValidator;
        this.adapter = adapter;
        this.inboundMessages$ = getObservable()
            .fromEvent(window, 'message')
            .filter(function (message) { return _this.messageValidator.validate(message); })
            .pluck('data');
        this.requests$ = this.messagesOfType('request');
        this.responses$ = this.messagesOfType('response');
        this.notifications$ = this.messagesOfType('notification');
        this.inboundMessages$.subscribe(function (_a) {
            var id = _a.id;
            return _this.messageFactory.invalidateID(id);
        });
    }
    // ------------------------------------------------------------------------------------------------
    // API
    // ------------------------------------------------------------------------------------------------
    /**
     * @param {string} channel
     * @param {*} payload
     *
     * @return {Observable<object>}
     * @public
     */
    Messenger.prototype.request = function (channel, payload) {
        var request = this.messageFactory.makeRequest(channel, payload);
        var responseObservable = this.createResponseObservable(request.id);
        this.adapter.postMessage(request);
        return responseObservable;
    };
    /**
     * Sends a notification through given channel to the remote window, carrying given payload.
     *
     * @param {string} channel
     * @param {*} payload
     * @return {Messenger}
     * @public
     */
    Messenger.prototype.notify = function (channel, payload) {
        this.adapter.postMessage(this.messageFactory.makeNotification(channel, payload));
        return this;
    };
    /**
     * Returns an Observable that emits the subset of inbound notification messages where their channel equals given
     * channel name.
     *
     * @param {string} channel
     * @return {Observable<object>}
     * @public
     */
    Messenger.prototype.requests = function (channel) {
        var _this = this;
        return this.requests$
            .filter(function (request) { return request.channel === channel; })
            .map(function (request) { return new RxPostmessengerRequest(request.id, request.channel, request.payload, function (payload) { return _this.respond(request.id, channel, payload); }); });
    };
    /**
     * Returns an Observable that emits the subset of inbound notification messages where their channel equals given
     * channel name.
     *
     * @param {string} channel
     * @return {Observable<*>}
     * @public
     */
    Messenger.prototype.notifications = function (channel) {
        return this.notifications$
            .filter(function (notification) { return notification.channel === channel; })
            .pluck('payload');
    };
    // ------------------------------------------------------------------------------------------------
    // Private runtime
    // ------------------------------------------------------------------------------------------------
    /**
     * Sends a response through given channel to the remote window, carrying given payload.
     *
     * @param {string} requestId
     * @param channel
     * @param {*} payload
     * @return {Messenger}
     * @private
     */
    Messenger.prototype.respond = function (requestId, channel, payload) {
        this.adapter.postMessage(this.messageFactory.makeResponse(requestId, channel, payload));
        return this;
    };
    /**
     * Creates an Observable that completes after a single emission of the MessageEvent response for request of given
     * requestId. If no matching response is ever received, the Observable never completes.
     *
     * @param {string} requestId
     * @return {Observable<object>}
     * @private
     */
    Messenger.prototype.createResponseObservable = function (requestId) {
        return this.responses$
            .filter(function (response) { return response.requestId === requestId; })
            .pluck('payload')
            .take(1);
    };
    /**
     * Returns an Observable emitting all inbound messages` of given type.
     *
     * Returns an Observable emitting a subset of MessageEvent objects emitted
     * by this.inboundMessages$, passing through all MessageEvent objects whose
     * data.type property matches given type.
     *
     * @param {string} type
     * @return {Observable<object>}
     * @private
     */
    Messenger.prototype.messagesOfType = function (type) {
        return this.inboundMessages$
            .filter(function (message) { return message.type === type; });
    };
    return Messenger;
}());
export { Messenger };
//# sourceMappingURL=Messenger.js.map